/*
    libmaus2
    Copyright (C) 2019 German Tischler-Höhle

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#if ! defined(LIBMAUS2_GTF_TRANSCRIPT_HPP)
#define LIBMAUS2_GTF_TRANSCRIPT_HPP

#include <libmaus2/util/NumberSerialisation.hpp>

namespace libmaus2
{
	namespace gtf
	{
		struct Transcript
		{
			uint64_t id_offset;
			uint64_t start;
			uint64_t end;
			bool strand;
			uint64_t gene_id;
			uint64_t c_start;
			uint64_t c_end;
			uint64_t id;

			Transcript() {}
			Transcript(uint64_t const rcstart, uint64_t const rcend)
			: c_start(rcstart), c_end(rcend) {}
			Transcript(
				uint64_t const rid_offset,
				uint64_t const rstart,
				uint64_t const rend,
				bool const rstrand,
				uint64_t const rgene_id,
				uint64_t const rc_start,
				uint64_t const rc_end,
				uint64_t const rid
			) : id_offset(rid_offset), start(rstart), end(rend), strand(rstrand), gene_id(rgene_id),
			    c_start(rc_start), c_end(rc_end), id(rid)
			{

			}

			int64_t getFrom() const
			{
				return static_cast<int64_t>(c_start)-1;
			}

			int64_t getTo() const
			{
				return c_end;
			}

			libmaus2::math::IntegerInterval<int64_t> getInterval() const
			{
				return libmaus2::math::IntegerInterval<int64_t>(getFrom(),getTo()-1);
			}

			std::ostream & serialise(std::ostream & out) const
			{
				libmaus2::util::NumberSerialisation::serialiseNumber(out,id_offset);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,start);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,end);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,strand);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,gene_id);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,c_start);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,c_end);
				libmaus2::util::NumberSerialisation::serialiseNumber(out,id);
				return out;
			}

			std::istream & deserialise(std::istream & in)
			{
				id_offset = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				start = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				end = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				strand = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				gene_id = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				c_start = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				c_end = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				id = libmaus2::util::NumberSerialisation::deserialiseNumber(in);
				return in;
			}

			std::ostream & print(
				std::ostream & out,
				libmaus2::autoarray::AutoArray<char> const & Aid,
				char const * chr,
				char const * gene_id,
				char const * gene_name) const
			{
				out
					<< chr
					<< "\t"
					<< "SOURCE"
					<< "\t"
					<< "transcript"
					<< "\t"
					<< c_start
					<< "\t"
					<< c_end
					<< "\t"
					<< "."
					<< "\t"
					<< (strand?"+":"-")
					<< "\t"
					<< "."
					<< "\t"
					<< "transcript_id: \"" << Aid.begin() + id_offset << "\"; "
					<< "gene_id: \"" << gene_id << "\"; "
					<< "gene_name: \"" << gene_name << "\";"
					<< "\n";

				return out;
			}
		};
	}
}
#endif
